@@ -24,7 +24,7 @@
 #  SUCH DAMAGE.
 #
 #  Created by Dmitry Karasik <dk@plab.ku.dk>
-#  $Id: Printer.pm,v 1.22 2006/01/06 19:13:29 dk Exp $
+#  $Id: Printer.pm,v 1.23 2008/04/09 20:10:41 dk Exp $
 #
 =pod
 
@@ -79,7 +79,7 @@ $unix = Prima::Application-> get_system_info-> {apc} == apc::Unix;
 
 use constant lpr  => 0;
 use constant file => 1;
-use constant exec => 2;
+use constant cmd  => 2;
 
 
 sub profile_default
@@ -145,7 +145,7 @@ sub init
 		if ( $unix) {
 			$self-> import_printers( 'printers', '/etc/printcap');
 			$self-> {printers}-> {GhostView} = deepcopy( $self-> {defaultData});
-			$self-> {printers}-> {GhostView}-> {spoolerType} = exec;
+			$self-> {printers}-> {GhostView}-> {spoolerType} = cmd;
 			$self-> {printers}-> {GhostView}-> {spoolerData} = 'gv -';
 		}
 		$self-> {printers}-> {File} = deepcopy( $self-> {defaultData});
@@ -628,7 +628,7 @@ sub init
 {
 	my $self = shift;
 	my %profile = $self-> SUPER::init(@_);
-	$self-> {data}-> {spoolerType} = Prima::PS::Printer::exec;
+	$self-> {data}-> {spoolerType} = Prima::PS::Printer::cmd;
 	$self-> {data}-> {spoolerData} = $profile{command};
 }
 
