@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  * 
- * $Id: codec_jpeg.c,v 1.12 2006/09/07 16:11:45 dk Exp $
+ * $Id: codec_jpeg.c,v 1.19 2008/04/28 09:58:27 dk Exp $
  *
  */
 
@@ -33,9 +33,17 @@
 #undef LOCAL
 #undef HAVE_STDDEF_H
 #undef HAVE_STDLIB_H
+#undef HAVE_BOOLEAN
+#ifdef BROKEN_PERL_PLATFORM
+#undef      FAR
+#undef      setjmp
+#undef      longjmp
+#define     setjmp _setjmp
+#endif
 #include <sys/types.h>
 #include <stdio.h>
 #include <jpeglib.h>
+#include <jerror.h>
 
 
 #ifdef __cplusplus
@@ -55,10 +63,7 @@ static ImgCodecInfo codec_info = {
    nil,    /* features  */
    "Prima::Image::jpeg",  /* module */
    "Prima::Image::jpeg",  /* package */
-   true,   /* canLoad */
-   false,  /* canLoadMultiple  */
-   true,   /* canSave */
-   false,  /* canSaveMultiple */
+   IMG_LOAD_FROM_FILE | IMG_LOAD_FROM_STREAM | IMG_SAVE_TO_FILE | IMG_SAVE_TO_STREAM,
    jpgbpp, /* save types */
    nil
 };
@@ -99,16 +104,122 @@ load_error_exit(j_common_ptr cinfo)
    longjmp( l-> j, 1);
 }
 
+/* begin ripoff from jdatasrc.c */
+typedef struct {
+  struct jpeg_source_mgr pub;	/* public fields */
+  JOCTET * buffer;		/* start of buffer */
+  boolean start_of_file;	/* have we gotten any data yet? */
+  ImgIORequest  *req;
+} my_source_mgr;
+
+typedef my_source_mgr * my_src_ptr;
+
+#define INPUT_BUF_SIZE  4096	/* choose an efficiently fread'able size */
+
+void
+init_source (j_decompress_ptr cinfo)
+{
+   ((my_src_ptr) cinfo->src)->start_of_file = true;
+}
+
+boolean
+fill_input_buffer (j_decompress_ptr cinfo)
+{
+  unsigned long nbytes;
+  my_src_ptr src = (my_src_ptr) cinfo->src;
+
+  nbytes = req_read( src->req, INPUT_BUF_SIZE, src->buffer);
+  if (nbytes <= 0) {
+    if (src->start_of_file)	/* Treat empty input file as fatal error */
+      ERREXIT(cinfo, JERR_INPUT_EMPTY);
+    WARNMS(cinfo, JWRN_JPEG_EOF);
+    /* Insert a fake EOI marker */
+    src->buffer[0] = (JOCTET) 0xFF;
+    src->buffer[1] = (JOCTET) JPEG_EOI;
+    nbytes = 2;
+  }
+
+  src->pub.next_input_byte = src->buffer;
+  src->pub.bytes_in_buffer = nbytes;
+  src->start_of_file = false;
+
+  return true;
+}
+
+void
+skip_input_data (j_decompress_ptr cinfo, long num_bytes)
+{
+  my_src_ptr src = (my_src_ptr) cinfo->src;
+
+  /* Just a dumb implementation for now.  Could use fseek() except
+   * it doesn't work on pipes.  Not clear that being smart is worth
+   * any trouble anyway --- large skips are infrequent.
+   */
+  if (num_bytes > 0) {
+    while (num_bytes > (long) src->pub.bytes_in_buffer) {
+      num_bytes -= (long) src->pub.bytes_in_buffer;
+      (void) fill_input_buffer(cinfo);
+      /* note we assume that fill_input_buffer will never return FALSE,
+       * so suspension need not be handled.
+       */
+    }
+    src->pub.next_input_byte += (size_t) num_bytes;
+    src->pub.bytes_in_buffer -= (size_t) num_bytes;
+  }
+}
+
+/*
+ * Terminate source --- called by jpeg_finish_decompress
+ * after all data has been read.  Often a no-op.
+ * NB: *not* called by jpeg_abort or jpeg_destroy; surrounding
+ * application must deal with any cleanup that should happen even
+ * for error exit.
+ */
+
+void
+term_source (j_decompress_ptr cinfo)
+{
+  /* no work necessary here */
+}
+
+
+static void
+custom_src( j_decompress_ptr cinfo, PImgLoadFileInstance fi)
+{
+  my_src_ptr src;
+
+  cinfo->src = (struct jpeg_source_mgr *) malloc(sizeof(my_source_mgr));
+  src = (void*) cinfo-> src;			 
+  src-> buffer = (JOCTET *) malloc( INPUT_BUF_SIZE * sizeof(JOCTET));
+  src-> pub.init_source = init_source;
+  src-> pub.fill_input_buffer = fill_input_buffer;
+  src-> pub.skip_input_data = skip_input_data;
+  src-> pub.resync_to_restart = jpeg_resync_to_restart; /* use default method */
+  src-> pub.term_source = term_source;
+  src-> pub.bytes_in_buffer = 0; /* forces fill_input_buffer on first read */
+  src-> pub.next_input_byte = NULL; /* until buffer loaded */
+
+  src-> req    = fi-> req;
+}
+/* end ripoff from jdatasrc.c */
+
 static void * 
 open_load( PImgCodec instance, PImgLoadFileInstance fi)
 {
    LoadRec * l;
-   Byte buf[4];
+   Byte buf[2];
+
+   if ( req_seek( fi-> req, 0, SEEK_SET) < 0) return false;
+   if ( req_read( fi-> req, 2, buf) < 0) {
+      req_seek( fi-> req, 0, SEEK_SET);
+      return false;
+   }
+   if ( memcmp( "\xff\xd8", buf, 2) != 0) {
+      req_seek( fi-> req, 0, SEEK_SET);
+      return false;   
+   }
+   if ( req_seek( fi-> req, 0, SEEK_SET) < 0) return false;
 
-   if ( fseek( fi-> f, 0, SEEK_SET) < 0) return false;
-   if ( fread( buf, 1, 2, fi-> f) != 2) return false;
-   if ( memcmp( "\xff\xd8", buf, 2) != 0) return false;   
-   if ( fseek( fi-> f, 0, SEEK_SET) < 0) return false;
    fi-> stop = true;
    fi-> frameCount = 1;
    
@@ -128,7 +239,7 @@ open_load( PImgCodec instance, PImgLoadFileInstance fi)
       return false;
    } 
    jpeg_create_decompress( &l-> d);
-   jpeg_stdio_src( &l-> d, fi-> f);
+   custom_src( &l-> d, fi);
    jpeg_read_header( &l-> d, true);
    l-> init = false;
    return l;
@@ -152,13 +263,14 @@ load( PImgCodec instance, PImgLoadFileInstance fi)
    if ( bpp == 8) bpp |= imGrayScale;
    CImage( fi-> object)-> create_empty( fi-> object, 1, 1, bpp);
    if ( fi-> noImageData) {
-      hv_store( fi-> frameProperties, "width",  5, newSViv( l-> d. output_width), 0);
-      hv_store( fi-> frameProperties, "height", 6, newSViv( l-> d. output_height), 0);
+      (void) hv_store( fi-> frameProperties, "width",  5, newSViv( l-> d. output_width), 0);
+      (void) hv_store( fi-> frameProperties, "height", 6, newSViv( l-> d. output_height), 0);
       jpeg_abort_decompress( &l-> d);
       return true;
    }   
    
    CImage( fi-> object)-> create_empty( fi-> object, l-> d. output_width, l-> d. output_height, bpp);
+   EVENT_HEADER_READY(fi);
    {
       Byte * dest = i-> data + ( i-> h - 1) * i-> lineSize;
       while ( l-> d.output_scanline < l-> d.output_height ) {
@@ -169,8 +281,10 @@ load( PImgCodec instance, PImgLoadFileInstance fi)
          if ( bpp == 24) 
             cm_reverse_palette(( PRGBColor) dest, ( PRGBColor) dest, i-> w);
          dest -= scanlines * i-> lineSize;
+         EVENT_TOPDOWN_SCANLINES_READY(fi,scanlines);
       }   
    }   
+   EVENT_SCANLINES_FINISHED(fi);
    jpeg_finish_decompress(&l-> d);
    return true;
 }   
@@ -180,6 +294,10 @@ static void
 close_load( PImgCodec instance, PImgLoadFileInstance fi)
 {
    LoadRec * l = ( LoadRec *) fi-> instance;
+   my_src_ptr src = (my_src_ptr) l->d.src;
+   free( src-> buffer);
+   free( src);
+   l->d.src = NULL;
    jpeg_destroy_decompress(&l-> d);
    free( l);
 }
@@ -222,6 +340,92 @@ save_defaults( PImgCodec c)
    return profile;
 }
 
+/* begin ripoff from jdatadst.c */
+
+#define OUTPUT_BUF_SIZE  4096	/* choose an efficiently fwrite'able size */
+
+/* Expanded data destination object for stdio output */
+
+typedef struct {
+  struct jpeg_destination_mgr pub; /* public fields */
+  PImgIORequest req;
+  JOCTET * buffer;		/* start of buffer */
+} my_destination_mgr;
+
+typedef my_destination_mgr * my_dest_ptr;
+
+void
+init_destination (j_compress_ptr cinfo)
+{
+  my_dest_ptr dest = (my_dest_ptr) cinfo->dest;
+
+  /* Allocate the output buffer --- it will be released when done with image */
+  dest->buffer = (JOCTET *) malloc( OUTPUT_BUF_SIZE * sizeof(JOCTET));
+  dest->pub.next_output_byte = dest->buffer;
+  dest->pub.free_in_buffer = OUTPUT_BUF_SIZE;
+}
+
+
+/* Empty the output buffer --- called whenever buffer fills up.  */
+
+boolean
+empty_output_buffer (j_compress_ptr cinfo)
+{
+  my_dest_ptr dest = (my_dest_ptr) cinfo->dest;
+
+  if ( req_write(dest->req, OUTPUT_BUF_SIZE, dest->buffer) !=
+      (size_t) OUTPUT_BUF_SIZE)
+    ERREXIT(cinfo, JERR_FILE_WRITE);
+
+  dest->pub.next_output_byte = dest->buffer;
+  dest->pub.free_in_buffer = OUTPUT_BUF_SIZE;
+
+  return true;
+}
+
+
+/*
+ * Terminate destination --- called by jpeg_finish_compress
+ * after all data has been written.  Usually needs to flush buffer.
+ *
+ * NB: *not* called by jpeg_abort or jpeg_destroy; surrounding
+ * application must deal with any cleanup that should happen even
+ * for error exit.
+ */
+
+void
+term_destination (j_compress_ptr cinfo)
+{
+  my_dest_ptr dest = (my_dest_ptr) cinfo->dest;
+  size_t datacount = OUTPUT_BUF_SIZE - dest->pub.free_in_buffer;
+
+  /* Write any data remaining in the buffer */
+  if (datacount > 0) {
+    if (req_write(dest->req, datacount, dest->buffer) != datacount)
+      ERREXIT(cinfo, JERR_FILE_WRITE);
+  }
+  req_flush(dest->req);
+  /* Make sure we wrote the output file OK */
+  if (req_error(dest->req))
+    ERREXIT(cinfo, JERR_FILE_WRITE);
+}
+
+void
+custom_dest(j_compress_ptr cinfo, PImgIORequest req)
+{
+  my_dest_ptr dest;
+
+  cinfo->dest = (struct jpeg_destination_mgr *) malloc( sizeof(my_destination_mgr));
+  dest = (my_dest_ptr) cinfo->dest;
+  dest->pub.init_destination = init_destination;
+  dest->pub.empty_output_buffer = empty_output_buffer;
+  dest->pub.term_destination = term_destination;
+  dest->req = req;
+}
+
+/* end ripoff from jdatadst.c */
+
+
 static void *
 open_save( PImgCodec instance, PImgSaveFileInstance fi)
 {
@@ -244,7 +448,7 @@ open_save( PImgCodec instance, PImgSaveFileInstance fi)
       return false;
    } 
    jpeg_create_compress( &l-> c);
-   jpeg_stdio_dest( &l-> c, fi-> f);
+   custom_dest( &l-> c, fi-> req);
    l-> init = false;
    return l;
 }
@@ -252,6 +456,7 @@ open_save( PImgCodec instance, PImgSaveFileInstance fi)
 static Bool   
 save( PImgCodec instance, PImgSaveFileInstance fi)
 {
+   dPROFILE;
    PImage i = ( PImage) fi-> object;
    SaveRec * l = ( SaveRec *) fi-> instance;
    HV * profile = fi-> objectExtras;
@@ -308,7 +513,11 @@ static void
 close_save( PImgCodec instance, PImgSaveFileInstance fi)
 {
    SaveRec * l = ( SaveRec *) fi-> instance;
+   my_dest_ptr dest = (my_dest_ptr) l->c.dest;
+   free( dest-> buffer);
+   free( dest);
    free( l-> buf);
+   l->c.dest = NULL;
    jpeg_destroy_compress(&l-> c);
    free( l);
 }
