@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  * 
- * $Id: img.c,v 1.22 2007/05/16 21:16:24 dk Exp $
+ * $Id: img.c,v 1.31 2008/04/28 09:58:27 dk Exp $
  *
  */
 /* Created by Dmitry Karasik <dk@plab.ku.dk> */
@@ -44,7 +44,6 @@ extern "C" {
 
 static Bool initialized = false;   
 
-
 void 
 apc_img_init( void)
 {
@@ -183,13 +182,48 @@ apc_img_profile_add( HV * to, HV * from, HV * keys)
          continue;
       holder = hv_fetch( from, key, keyLen, 0);
       if ( holder) 
-         hv_store( to, key, keyLen, newSVsv( *holder), 0);
+         (void) hv_store( to, key, keyLen, newSVsv( *holder), 0);
    }
 }   
 
-PList 
-apc_img_load( Handle self, char * fileName, HV * profile, char * error)
+static size_t 
+stdio_read( void * f, size_t bufsize, void * buffer)
 {
+    return fread( buffer, 1, bufsize, ( FILE*) f);
+}
+
+static size_t 
+stdio_write( void * f, size_t bufsize, void * buffer)
+{
+    return fwrite( buffer, 1, bufsize, ( FILE*) f);
+}
+
+static int
+stdio_seek( void * f, long offset, int whence)
+{
+    return fseek( ( FILE*) f, offset, whence);
+}
+
+static long
+stdio_tell( void * f)
+{
+    return ftell( ( FILE*) f);
+}
+
+
+static ImgIORequest std_ioreq = {
+      stdio_read,
+      stdio_write,
+      stdio_seek,
+      stdio_tell,
+      (void*) fflush,
+      (void*) ferror
+};
+
+PList 
+apc_img_load( Handle self, char * fileName, PImgIORequest ioreq,  HV * profile, char * error)
+{
+   dPROFILE;
    int i, profiles_len = 0, lastFrame = -2, codecID = -1;
    PList ret;
    PImgCodec c = nil;
@@ -201,6 +235,8 @@ apc_img_load( Handle self, char * fileName, HV * profile, char * error)
    Bool incrementalLoad = false;
    Bool iconUnmask = false;
    char * baseClassName = "Prima::Image";
+   ImgIORequest sioreq;
+   int  load_mask;
 
 
 #define out(x){ err = true;\
@@ -220,8 +256,18 @@ apc_img_load( Handle self, char * fileName, HV * profile, char * error)
    fi. errbuf = error;
 
    /* open file */
-   if (( fi. f = ( FILE *) fopen( fileName, "rb")) == NULL)
-      out( strerror( errno));
+   if ( ioreq == NULL) {
+      memcpy( &sioreq, &std_ioreq, sizeof( sioreq));
+      if (( sioreq. handle = fopen( fileName, "rb")) == NULL)
+         out( strerror( errno));
+      fi. req = &sioreq;
+      fi. req_is_stdio = true;
+      load_mask = IMG_LOAD_FROM_FILE;
+   } else {
+      fi. req = ioreq;
+      fi. req_is_stdio = false;
+      load_mask = IMG_LOAD_FROM_STREAM;
+   }
    fi. fileName = fileName;
    fi. stop = false;
 
@@ -267,7 +313,16 @@ apc_img_load( Handle self, char * fileName, HV * profile, char * error)
    
    if ( pexist( iconUnmask) && pget_B( iconUnmask))
       fi. iconUnmask = iconUnmask = true;
-
+   
+   if ( pexist( eventMask))
+      fi. eventMask = pget_i( eventMask);
+   
+   if ( pexist( eventDelay))
+      fi. eventDelay = 1000.0 * pget_f( eventDelay);
+   if ( fi. eventDelay <= 0)
+      fi. eventDelay = 100; /* 100 ms. reasonable? */
+   EVENT_SCANLINES_RESET(&fi);
+   
    if ( pexist( profiles)) {
       SV * sv = pget_sv( profiles);
       if ( SvOK( sv) && SvROK( sv) && SvTYPE( SvRV( sv)) == SVt_PVAV) {
@@ -296,7 +351,6 @@ apc_img_load( Handle self, char * fileName, HV * profile, char * error)
    /* finding codec */
    {
       Bool * loadmap = ( Bool *) malloc( sizeof( Bool) * imgCodecs. count);
-      char * xc = fileName + strlen( fileName);
 
       if ( !loadmap) 
          out("Not enough memory");
@@ -310,54 +364,55 @@ apc_img_load( Handle self, char * fileName, HV * profile, char * error)
             continue;
          } 
       }
-      
-      while ( xc != fileName) {
-         if ( *xc == '.') 
-            break;
-         xc--;
-      }   
-      if ( *xc == '.') xc++;
-
-      /* finding by extension first */
       c = nil;
-      for ( i = 0; i < imgCodecs. count; i++) {
-         int j = 0, found = false;
-         if ( loadmap[ i]) continue;
-         c = ( PImgCodec ) ( imgCodecs. items[ i]);
-         while ( c-> info-> fileExtensions[ j]) {
-            if ( stricmp( c-> info-> fileExtensions[ j], xc) == 0) {
-               found = true;
-               break;
+      
+      /* finding by extension first */
+      if ( fileName) {
+         int fileNameLen = strlen( fileName);
+         for ( i = 0; i < imgCodecs. count; i++) {
+            int j = 0, found = false;
+            if ( loadmap[ i]) continue;
+            c = ( PImgCodec ) ( imgCodecs. items[ i]);
+            while ( c-> info-> fileExtensions[ j]) {
+               char * ext = c-> info-> fileExtensions[ j];
+               int extLen = strlen( ext);
+               if ( extLen < fileNameLen && stricmp( fileName + fileNameLen - extLen, ext) == 0) {
+                  found = true;
+                  break;
+               }   
+               j++;
+            } 
+            if ( found) {
+               loadmap[ i] = true;
+
+               if ( !( c-> info-> IOFlags & load_mask)) {
+	          c = nil;
+                  continue;
+	       }	  
+               if (( fi. instance = c-> vmt-> open_load( c, &fi)) != NULL) {
+                  codecID = i;
+                  break;
+               }   
+
+               if ( fi. stop) { 
+                  err = true; 
+                  free( loadmap);
+                  goto EXIT_NOW; 
+               }
             }   
-            j++;
-         } 
-         if ( found) {
-            loadmap[ i] = true;
-
-            if ( !c-> info-> canLoad)
-               continue;
-
-            if (( fi. instance = c-> vmt-> open_load( c, &fi)) != NULL) {
-               codecID = i;
-               break;
-            }   
-
-            if ( fi. stop) { 
-               err = true; 
-               free( loadmap);
-               goto EXIT_NOW; 
-            }
-         }   
-         c = nil;
-      }   
+            c = nil;
+         }
+      }
       
       /* use first suitable codec */
       if ( c == nil) {
-         for ( i = 0; i < imgCodecs. count; i++) {        
+         for ( i = 0; i < imgCodecs. count; i++) {
             if ( loadmap[ i]) continue;
             c = ( PImgCodec ) ( imgCodecs. items[ i]);
-            if ( !c-> info-> canLoad)
+            if ( !( c-> info-> IOFlags & load_mask)) {
+	          c = nil;
                   continue;
+            }
             if (( fi. instance = c-> vmt-> open_load( c, &fi)) != NULL) {
                codecID = i;
                break;
@@ -372,7 +427,7 @@ apc_img_load( Handle self, char * fileName, HV * profile, char * error)
       }
       free( loadmap);
       if ( !c) out("No appropriate codec found");
-   }   
+   }
 
    if ( fi. loadAll) {
       if ( fi. frameCount >= 0) {
@@ -387,6 +442,7 @@ apc_img_load( Handle self, char * fileName, HV * profile, char * error)
       }   
    }   
 
+
    /* use common profile */
    def = c-> vmt-> load_defaults( c); 
    commonHV = newHV();
@@ -396,7 +452,7 @@ apc_img_load( Handle self, char * fileName, HV * profile, char * error)
    }
 
    if ( fi. loadExtras && c-> info-> fileType) 
-      hv_store( fi. fileProperties, "codecID", 7, newSViv( codecID), 0);
+      (void) hv_store( fi. fileProperties, "codecID", 7, newSViv( codecID), 0);
 
    /* loading */
    for ( i = 0; i < fi. frameMapSize; i++) {
@@ -405,8 +461,8 @@ apc_img_load( Handle self, char * fileName, HV * profile, char * error)
 
       fi. frame = incrementalLoad ? i : fi. frameMap[ i];
       if (( fi. frameCount >= 0 && fi. frame >= fi. frameCount) || 
-          ( !c-> info-> canLoadMultiple && fi. frame > 0)) { 
-         if ( !c-> info-> canLoadMultiple && fi. frameCount < 0)
+          ( !(c-> info-> IOFlags & IMG_LOAD_MULTIFRAME) && fi. frame > 0)) { 
+         if ( !(c-> info-> IOFlags & IMG_LOAD_MULTIFRAME) && fi. frameCount < 0)
             fi. frameCount = i; 
          if ( incrementalLoad) 
             /* that means, codec bothered to set frameCount at last - report no error then */
@@ -526,20 +582,20 @@ apc_img_load( Handle self, char * fileName, HV * profile, char * error)
          apc_img_profile_add( extras, fi. fileProperties,  fi. fileProperties);
          apc_img_profile_add( extras, fi. frameProperties, fi. frameProperties);
          if ( i == 0) firstObjectExtras = extras; 
-         hv_store(( HV* )SvRV((( PAnyObject) fi. object)-> mate), "extras", 6, newSVsv( sv), 0);
+         (void) hv_store(( HV* )SvRV((( PAnyObject) fi. object)-> mate), "extras", 6, newSVsv( sv), 0);
          sv_free( sv);
       } else if ( fi. noImageData) { /* no extras, report dimensions only */
          HV * extras = newHV();
          SV * sv = newRV_noinc(( SV *) extras), **item;
          if (( item = hv_fetch( fi. frameProperties, "width", 5, 0)) && SvOK( *item)) 
-            hv_store( extras, "width", 5, newSVsv( *item), 0);
+            (void) hv_store( extras, "width", 5, newSVsv( *item), 0);
          else
-            hv_store( extras, "width", 5, newSViv(PImage(fi.object)-> w), 0);
+            (void) hv_store( extras, "width", 5, newSViv(PImage(fi.object)-> w), 0);
          if (( item = hv_fetch( fi. frameProperties, "height", 6, 0)) && SvOK( *item)) 
-            hv_store( extras, "height", 6, newSVsv( *item), 0);
+            (void) hv_store( extras, "height", 6, newSVsv( *item), 0);
          else
-            hv_store( extras, "height", 6, newSViv(PImage(fi.object)-> h), 0);
-         hv_store(( HV* )SvRV((( PAnyObject) fi. object)-> mate), "extras", 6, newSVsv( sv), 0);
+            (void) hv_store( extras, "height", 6, newSViv(PImage(fi.object)-> h), 0);
+         (void) hv_store(( HV* )SvRV((( PAnyObject) fi. object)-> mate), "extras", 6, newSVsv( sv), 0);
          sv_free( sv);
       }
 
@@ -557,16 +613,18 @@ apc_img_load( Handle self, char * fileName, HV * profile, char * error)
       SV * sv = newRV_noinc(( SV *) extras);
       apc_img_profile_add( extras, fi. fileProperties,  fi. fileProperties);
       firstObjectExtras = extras; 
-      hv_store(( HV* )SvRV((( PAnyObject) self)-> mate), "extras", 6, newSVsv( sv), 0);
+      (void) hv_store(( HV* )SvRV((( PAnyObject) self)-> mate), "extras", 6, newSVsv( sv), 0);
       sv_free( sv);
    }   
    
 EXIT_NOW:;
    if ( fi. frameCount < 0 && pexist( wantFrames) && pget_i( wantFrames)) {
-      fi. frameCount = apc_img_frame_count( fileName);
+      if ( ioreq != NULL) 
+         req_seek( ioreq, 0, SEEK_SET);
+      fi. frameCount = apc_img_frame_count( fileName, ioreq);
    }
    if ( firstObjectExtras)
-      hv_store( firstObjectExtras, "frames", 6, newSViv( fi. frameCount), 0);
+      (void) hv_store( firstObjectExtras, "frames", 6, newSViv( fi. frameCount), 0);
    if ( err && ret)
       list_add( ret, nilHandle); /* indicate the error */
    if ( def)
@@ -575,26 +633,39 @@ EXIT_NOW:;
       sv_free(( SV *) commonHV);
    if ( fi. fileProperties) 
       sv_free((SV *) fi. fileProperties); 
-   if ( fi. f) 
-      fclose( fi. f);
+   if ( ioreq == NULL && fi.req != NULL && fi. req-> handle != NULL) 
+      fclose(( FILE*) fi. req-> handle);
    free( fi. frameMap);
    return ret;
 #undef out   
 #undef outd
-}   
+}
 
 int 
-apc_img_frame_count( char * fileName)
+apc_img_frame_count( char * fileName, PImgIORequest ioreq )
 {
    PImgCodec c = nil;
    ImgLoadFileInstance fi;
    int i, frameMap, ret = 0;
    char error[256];
+   ImgIORequest sioreq;
+   int load_mask;
 
    CHK;
    memset( &fi, 0, sizeof( fi));
    /* open file */
-   if (( fi. f = ( FILE *) fopen( fileName, "rb")) == NULL) goto EXIT_NOW;
+   if ( ioreq == NULL) {
+      memcpy( &sioreq, &std_ioreq, sizeof( sioreq));
+      if (( sioreq. handle = fopen( fileName, "rb")) == NULL)
+         goto EXIT_NOW;
+      fi. req = &sioreq;
+      fi. req_is_stdio = true;
+      load_mask = IMG_LOAD_FROM_FILE;
+   } else {
+      fi. req = ioreq;
+      fi. req_is_stdio = false;
+      load_mask = IMG_LOAD_FROM_STREAM;
+   }
    
    /* assigning request */
    fi. fileName = fileName;
@@ -612,7 +683,6 @@ apc_img_frame_count( char * fileName)
    /* finding codec */
    {
       Bool * loadmap = ( Bool*) malloc( sizeof( Bool) * imgCodecs. count);
-      char * xc = fileName + strlen( fileName);
 
       if ( !loadmap) 
          return 0;
@@ -627,55 +697,58 @@ apc_img_frame_count( char * fileName)
          } 
       }
       
-      while ( xc != fileName) {
-         if ( *xc == '.') 
-            break;
-         xc--;
-      }   
-      if ( *xc == '.') xc++;
+      c = nil;
 
       /* finding by extension first */
-      c = nil;
-      for ( i = 0; i < imgCodecs. count; i++) {
-         int j = 0, found = false;
-         if ( loadmap[ i]) continue;
-         c = ( PImgCodec ) ( imgCodecs. items[ i]);
-         while ( c-> info-> fileExtensions[ j]) {
-            if ( stricmp( c-> info-> fileExtensions[ j], xc) == 0) {
-               found = true;
-               break;
-            }   
-            j++;
-         } 
-         if ( found) {
-            loadmap[ i] = true;
+      if ( fileName) {
+         int fileNameLen = strlen( fileName);
+         for ( i = 0; i < imgCodecs. count; i++) {
+            int j = 0, found = false;
+            if ( loadmap[ i]) continue;
+            c = ( PImgCodec ) ( imgCodecs. items[ i]);
+            while ( c-> info-> fileExtensions[ j]) {
+               char * ext = c-> info-> fileExtensions[ j];
+               int extLen = strlen( ext);
+               if ( extLen < fileNameLen && stricmp( fileName + fileNameLen - extLen, ext) == 0) {
+                  found = true;
+                  break;
+               }   
+               j++;
+            } 
+            if ( found) {
+               loadmap[ i] = true;
 
-            if ( !c-> info-> canLoad)
-               continue;
-            if (( fi. instance = c-> vmt-> open_load( c, &fi)) != NULL)
-               break;
-            
-            if ( fi. stop) { 
-               free( loadmap);
-               goto EXIT_NOW; 
-            }
-         }   
-         c = nil;
-      }   
+               if ( !( c-> info-> IOFlags & load_mask)) {
+	          c = nil;
+                  continue;
+	       }
+               if (( fi. instance = c-> vmt-> open_load( c, &fi)) != NULL)
+                  break;
+               
+               if ( fi. stop) { 
+                  free( loadmap);
+                  goto EXIT_NOW; 
+               }
+            }   
+            c = nil;
+         }
+      }
       
       if ( c == nil) {
          for ( i = 0; i < imgCodecs. count; i++) {        
             if ( loadmap[ i]) continue;
             c = ( PImgCodec ) ( imgCodecs. items[ i]);
-            if ( !c-> info-> canLoad)
+            if ( !( c-> info-> IOFlags & load_mask)) {
+	          c = nil;
                   continue;
+	    }  
             if (( fi. instance = c-> vmt-> open_load( c, &fi)) != NULL)
                break;
             if ( fi. stop) { 
                free( loadmap);
                goto EXIT_NOW; 
             }
-           c = nil;
+            c = nil;
          }
       }
       free( loadmap);
@@ -690,7 +763,7 @@ apc_img_frame_count( char * fileName)
       goto EXIT_NOW;
    }  
 
-   if ( !c-> info-> canLoadMultiple) {
+   if ( !( c-> info-> IOFlags & IMG_LOAD_MULTIFRAME)) {
       c-> vmt-> close_load( c, &fi);
       ret = 1; /* single-framed file. what else? */
       goto EXIT_NOW;
@@ -720,7 +793,7 @@ apc_img_frame_count( char * fileName)
    for ( i = 0; i < INT_MAX; i++) {
       fi. jointFrame = i > 0;
       frameMap = fi. frame = i;
-      if ( !c-> info-> canLoadMultiple) {
+      if ( !( c-> info-> IOFlags & IMG_LOAD_MULTIFRAME)) {
          c-> vmt-> close_load( c, &fi);
          if ( !( fi. instance = c-> vmt-> open_load( c, &fi))) {
             ret = i;
@@ -745,14 +818,15 @@ EXIT_NOW:;
       sv_free(( SV *) fi. frameProperties);
    if ( fi. fileProperties)
       sv_free(( SV *) fi. fileProperties);
-   if ( fi. f) 
-      fclose( fi. f);
+   if ( ioreq == NULL && fi.req != NULL && fi. req-> handle != NULL)
+      fclose(( FILE*) fi. req-> handle);
    return ret;
 }   
 
 int
-apc_img_save( Handle self, char * fileName, HV * profile, char * error)
+apc_img_save( Handle self, char * fileName, PImgIORequest ioreq, HV * profile, char * error)
 {
+   dPROFILE;
    int i;
    PImgCodec c = nil;
    ImgSaveFileInstance fi;
@@ -763,6 +837,8 @@ apc_img_save( Handle self, char * fileName, HV * profile, char * error)
    int xself = self ? 1 : 0;
    int ret = 0;
    Bool autoConvert = true;
+   ImgIORequest sioreq;
+   int save_mask;
 
 #define out(x){ err = true;\
    strncpy( error, x, 256);\
@@ -782,15 +858,25 @@ apc_img_save( Handle self, char * fileName, HV * profile, char * error)
       autoConvert = pget_B( autoConvert);
 
    /* open file */
-   if ( fi. append) {
+   if ( fi. append && ioreq == NULL) {
       FILE * f = ( FILE *) fopen( fileName, "rb");
       fclose( f);
       if ( !f) fi. append = false;
    }   
    
    fi. errbuf = error;
-   if (( fi. f = ( FILE *) fopen( fileName, fi. append ? "rb+" : "wb+" )) == NULL)
-      out( strerror( errno));
+   if ( ioreq == NULL) {
+      memcpy( &sioreq, &std_ioreq, sizeof( sioreq));
+      if (( sioreq. handle = fopen( fileName, fi. append ? "rb+" : "wb+" )) == NULL)
+         out( strerror( errno));
+      fi. req = &sioreq;
+      fi. req_is_stdio = true;
+      save_mask = IMG_SAVE_TO_FILE;
+   } else {
+      fi. req = ioreq;
+      fi. req_is_stdio = false;
+      save_mask = IMG_SAVE_TO_STREAM;
+   }
 
    fi. fileName     = fileName;
    
@@ -843,7 +929,6 @@ apc_img_save( Handle self, char * fileName, HV * profile, char * error)
    strcpy( error, "No appropriate codec found");
    {
       Bool * savemap = ( Bool*) malloc( sizeof( Bool) * imgCodecs. count);
-      char * xc = fileName + strlen( fileName);
 
       if ( !savemap)
          out("Not enough memory");
@@ -887,12 +972,13 @@ apc_img_save( Handle self, char * fileName, HV * profile, char * error)
              out("Codec index out of range");
 
          c = ( PImgCodec ) ( imgCodecs. items[ codecID]);
-
-         if ( !c-> info-> canSave) 
-            out("Codec cannot save images");
+         if ( !( c-> info-> IOFlags & save_mask))
+               out( ioreq ? 
+	          "Codec cannot save images to streams" : 
+		  "Codec cannot save images");
          
          if (( fi. append || ( fi. frameMapSize > 1)) && 
-               !c-> info-> canSaveMultiple) 
+               !( c-> info-> IOFlags & IMG_SAVE_MULTIFRAME)) 
             out("Codec cannot append frames");
          
          if (( fi. instance = c-> vmt-> open_save( c, &fi)) == NULL) 
@@ -915,21 +1001,17 @@ apc_img_save( Handle self, char * fileName, HV * profile, char * error)
          }      
       }   
      
-      if ( !c) {
+      if ( !c && fileName) {
+         int fileNameLen = strlen( fileName);
          /* finding codec by extension  */
-         while ( xc != fileName) {
-            if ( *xc == '.') 
-               break;
-            xc--;
-         }   
-         if ( *xc == '.') xc++;
-         
          for ( i = 0; i < imgCodecs. count; i++) {
             int j = 0, found = false;
             if ( savemap[ i]) continue;
             c = ( PImgCodec ) ( imgCodecs. items[ i]);
             while ( c-> info-> fileExtensions[ j]) {
-               if ( stricmp( c-> info-> fileExtensions[ j], xc) == 0) {
+	       char * ext = c-> info-> fileExtensions[ j];
+	       int extLen = strlen( ext);
+	       if ( extLen < fileNameLen && stricmp( fileName + fileNameLen - extLen, ext) == 0) {
                   found = true;
                   break;
                }   
@@ -938,12 +1020,16 @@ apc_img_save( Handle self, char * fileName, HV * profile, char * error)
 
             if ( found) {
                savemap[ i] = true;
-               if ( !c-> info-> canSave) 
+               if ( !( c-> info-> IOFlags & save_mask)) {
+	          c = nil;
                   continue;
+	       }
                
                if (( fi. append || ( fi. frameMapSize > 1)) 
-                   && !c-> info-> canSaveMultiple)
+                   && ( c-> info-> IOFlags & IMG_SAVE_MULTIFRAME)) {
+	          c = nil;
                   continue;
+	       }
                
                if ( !autoConvert) {
                   int j, *k = c-> info-> saveTypes, ok = 0;
@@ -957,13 +1043,15 @@ apc_img_save( Handle self, char * fileName, HV * profile, char * error)
                         k++;
                      }   
                   }   
-                  if ( !ok) 
+                  if ( !ok) {
+	             c = nil;
                      continue;
-               }      
+		  }
+               }
               
                if (( fi. instance = c-> vmt-> open_save( c, &fi)) != NULL)
                   break;
-            }   
+            }
             c = nil;
          }   
       }
@@ -975,7 +1063,7 @@ apc_img_save( Handle self, char * fileName, HV * profile, char * error)
       }   
    }   
 
-   snprintf( error, 256, "Error saving %s", fileName);
+   snprintf( error, 256, "Error saving %s", fileName ? fileName : "to stream");
 
    /* use common profile */
    def = c-> vmt-> save_defaults( c); 
@@ -1064,9 +1152,9 @@ apc_img_save( Handle self, char * fileName, HV * profile, char * error)
    
 EXIT_NOW:;
    free( fi. frameMap);
-   if ( fi. f)
-      fclose( fi. f);
-   if ( err)
+   if ( ioreq == NULL && fi. req-> handle != NULL)
+      fclose(( FILE*) fi. req-> handle);
+   if ( err && fileName)
       unlink( fileName);
    if ( def)
       sv_free(( SV *) def);
@@ -1145,7 +1233,7 @@ static AV * fill_plist( char * key, char ** list, HV * profile)
       av_push( av, newSVpv( *list, 0));
       list++;
    }   
-   hv_store( profile, key, strlen( key), newRV_noinc(( SV *) av), 0);
+   (void) hv_store( profile, key, strlen( key), newRV_noinc(( SV *) av), 0);
    return av;
 }  
 
@@ -1157,7 +1245,7 @@ static void fill_ilist( char * key, int * list, HV * profile)
       av_push( av, newSViv( *list));
       list++;
    }   
-   hv_store( profile, key, strlen( key), newRV_noinc(( SV *) av), 0);
+   (void) hv_store( profile, key, strlen( key), newRV_noinc(( SV *) av), 0);
 }   
 
 
@@ -1188,45 +1276,46 @@ apc_img_info2hash( PImgCodec codec)
    fill_plist( "featuresSupported", c-> featuresSupported, profile);
    pset_c( module,  c-> primaModule);
    pset_c( package, c-> primaPackage);
-   pset_i( canLoad,         c-> canLoad);
-   pset_i( canSave        , c-> canSave);
-   pset_i( canLoadMultiple, c-> canLoadMultiple);
-   pset_i( canSaveMultiple, c-> canSaveMultiple);
+   pset_i( canLoad,         c-> IOFlags & IMG_LOAD_FROM_FILE);
+   pset_i( canLoadStream  , c-> IOFlags & IMG_LOAD_FROM_STREAM);
+   pset_i( canLoadMultiple, c-> IOFlags & IMG_LOAD_MULTIFRAME);
+   pset_i( canSave        , c-> IOFlags & IMG_SAVE_TO_FILE);
+   pset_i( canSaveStream  , c-> IOFlags & IMG_SAVE_TO_STREAM);
+   pset_i( canSaveMultiple, c-> IOFlags & IMG_SAVE_MULTIFRAME);
    
-   
-   fill_ilist( "types",  c-> canSave ? c-> saveTypes : nil, profile);
+   fill_ilist( "types",  c-> saveTypes, profile);
 
-   if ( c-> canLoad) {
+   if ( c-> IOFlags & ( IMG_LOAD_FROM_FILE|IMG_LOAD_FROM_STREAM)) {
       hv = codec-> vmt-> load_defaults( codec);
-      if ( c-> canLoadMultiple) {
-         hv_store( hv, "index",        5, newSViv(0),     0);
-         hv_store( hv, "map",          3, newSVsv(nilSV), 0);
-         hv_store( hv, "loadAll",      7, newSViv(0),     0);
-         hv_store( hv, "wantFrames",  10, newSViv(0),     0);
+      if ( c-> IOFlags & IMG_LOAD_MULTIFRAME) {
+         (void) hv_store( hv, "index",        5, newSViv(0),     0);
+         (void) hv_store( hv, "map",          3, newSVsv(nilSV), 0);
+         (void) hv_store( hv, "loadAll",      7, newSViv(0),     0);
+         (void) hv_store( hv, "wantFrames",  10, newSViv(0),     0);
       }
-      hv_store( hv, "loadExtras",  10, newSViv(0),     0);
-      hv_store( hv, "noImageData", 11, newSViv(0),     0);
-      hv_store( hv, "iconUnmask",  10, newSViv(0),     0);
-      hv_store( hv, "className",    9, newSVpv("Prima::Image", 0), 0);
+      (void) hv_store( hv, "loadExtras",  10, newSViv(0),     0);
+      (void) hv_store( hv, "noImageData", 11, newSViv(0),     0);
+      (void) hv_store( hv, "iconUnmask",  10, newSViv(0),     0);
+      (void) hv_store( hv, "className",    9, newSVpv("Prima::Image", 0), 0);
    } else
       hv = newHV();
    pset_sv_noinc( loadInput, newRV_noinc(( SV *) hv));
    
-   av = fill_plist( "loadOutput", c-> canLoad ? c-> loadOutput : nil, profile);
-   if ( c-> canLoad) {
-      if ( c-> canLoadMultiple) 
+   av = fill_plist( "loadOutput", c-> loadOutput, profile);
+   if ( c-> IOFlags & ( IMG_LOAD_FROM_FILE|IMG_LOAD_FROM_STREAM)) {
+      if ( c-> IOFlags & IMG_LOAD_MULTIFRAME) 
          av_push( av, newSVpv( "frames", 0));
       av_push( av, newSVpv( "height", 0));
       av_push( av, newSVpv( "width",  0));
       av_push( av, newSVpv( "codecID", 0));      
    }
    
-   if ( c-> canSave) {
+   if ( c-> IOFlags & ( IMG_SAVE_TO_FILE|IMG_SAVE_TO_STREAM)) {
       hv = codec-> vmt-> save_defaults( codec);
-      if ( c-> canSaveMultiple)
-         hv_store( hv, "append",       6, newSViv(0), 0);
-      hv_store( hv, "autoConvert", 10, newSViv(1), 0);
-      hv_store( hv, "codecID",     7,  newSVsv( nilSV), 0);
+      if ( c-> IOFlags & IMG_SAVE_MULTIFRAME) 
+         (void) hv_store( hv, "append",       6, newSViv(0), 0);
+      (void) hv_store( hv, "autoConvert", 10, newSViv(1), 0);
+      (void) hv_store( hv, "codecID",     7,  newSVsv( nilSV), 0);
    } else
       hv = newHV();
    pset_sv_noinc( saveInput, newRV_noinc(( SV *) hv));
@@ -1236,7 +1325,41 @@ apc_img_info2hash( PImgCodec codec)
 char * imgPVEmptySet[] = { nil };
 int    imgIVEmptySet[] = { 0 };
 
+void
+apc_img_notify_header_ready( PImgLoadFileInstance fi)
+{
+      Event e = { cmImageHeaderReady };
+      CImage( fi-> object)-> message( fi-> object, &e);
+}
 
+void
+apc_img_notify_scanlines_ready( PImgLoadFileInstance fi, int scanlines)
+{
+      Event e;
+      int height;
+      unsigned int dt;
+      struct timeval t;
+
+      fi-> lastCachedScanline += scanlines;
+      gettimeofday( &t, nil);
+      dt = 
+         t.tv_sec * 1000 + t.tv_usec / 1000 -
+         fi-> lastEventTime.tv_sec * 1000 - fi-> lastEventTime.tv_usec / 1000;
+
+      if ( dt < fi-> eventDelay) return;
+      if ( fi-> lastEventScanline == fi-> lastCachedScanline) return;
+
+      e. cmd = cmImageDataReady;
+      height = PImage( fi-> object)-> h;
+      e. gen. R. left   = 0;
+      e. gen. R. right  = PImage( fi-> object)-> w - 1;
+      e. gen. R. top    = height - fi-> lastEventScanline  - 1;
+      e. gen. R. bottom = height - fi-> lastCachedScanline;
+      CImage( fi-> object)-> message( fi-> object, &e);
+
+      gettimeofday( &fi-> lastEventTime, nil);
+      fi-> lastEventScanline = fi-> lastCachedScanline;
+}
 #ifdef __cplusplus
 }
 #endif
